--[[
	KillingSpree2 authored by Douglas Craig (chexsum) <chexsum@gmail.com>

	Announces consecutive killing blows. 

]]
KillingSpree = LibStub("AceAddon-3.0"):NewAddon("KillingSpree", "AceEvent-3.0", "AceTimer-3.0", "AceConsole-3.0", "LibSink-2.0")

function KillingSpree:OnInitialize()
	self.scheme = {}
	self.schemes = { "UT2k3", "UT2k4Male", "UT2k4Male2", "UT2k4Female", "UT2k4Female2" }
	for scheme = 1, #self.schemes do
		self.scheme[self.schemes[scheme]] = {
			['headshot'] = { "headshot.wav", "Headshot!" },
			['firstblood'] = { "firstblood.wav", "First Blood!" },
			['youhavewon'] = { "youhavewon.wav", "You have won the match!" },
			['youhavelost'] = { "youhavelost.wav", "You have lost the match!" },
			['multi'] = {
				[2] = { "doublekill.wav", "Double Kill!" },
				[3] = { "multikill.wav", "Multi Kill!" },
				[4] = { "megakill.wav", "Mega Kill!" },
				[5] = { "ultrakill.wav", "Ultra Kill!" },
				[6] = { "monsterkill.wav", "Monster Kill!" },
			},
			['spree'] = {
				[10] = { "killingspree.wav", "Killing Spree!" },
				[15] = { "rampage.wav", "Rampage!" },
				[20] = { "dominating.wav", "Dominating!" },
				[25] = { "unstoppable.wav", "Unstoppable!" },
				[30] = { "godlike.wav", "God-like!" },
			},
			['count'] = {
				[1] = { "1.wav", "One!" },
				[2] = { "2.wav", "Two!" },
				[3] = { "3.wav", "Three!" },
				[4] = { "4.wav", "Four!" },
				[5] = { "5.wav", "Five!" },
			},
		}
		if self.schemes[scheme] ~= "UT2k3" then
			self.scheme[self.schemes[scheme]]['humiliation'] = { "humiliation.wav", "Humiliating defeat!" }
			self.scheme[self.schemes[scheme]]['flawless'] = { "flawless.wav", "Flawless victory!" }
			self.scheme[self.schemes[scheme]]['multi'][7] = { "ludicrouskill.wav", "Ludicrous Kill!" }
			self.scheme[self.schemes[scheme]]['spree'][35] = { "wickedsick.wav", "Wicked Sick!" }
		end
	end
	if UnitSex('player') == 3 then
		self.defaultscheme = "UT2k4Female"
	else
		self.defaultscheme = "UT2k4Male"
	end
	self.defaults = {
		profile = {
			battleground = true,
			instance = true,
			death = true,
			sound = true,
			flash = true,
			slain = true,
			pvp = true,
			scheme = self.defaultscheme,
		},
	}
	self.db = LibStub("AceDB-3.0"):New("KillingSpreeDB", self.defaults, "Default")
	self.options = {
		type = 'group',
		name = "KillingSpree",
		desc = "Consecutive kill announcer",
		icon = "Interface\\AddOns\\KillingSpree\\icon",
		args = {
			scheme = {
				order = 1,
				type = 'select',
				name = "Sound scheme",
				desc = "Selects the sound scheme",
				get = function(info)
					return self.db.profile.scheme
				end,
				set = function(info, value)
					self.db.profile.scheme = value
					self:Announce({ string.format("%s sound scheme selected", self.schemes[self.db.profile.scheme]), string.format("Interface\\AddOns\\KillingSpree\\Snds\\%s\\%s", self.schemes[self.db.profile.scheme], self.scheme[self.schemes[self.db.profile.scheme]]['spree'][10][1]), true })
				end,
				values = self.schemes,
			},
			battleground = {
				order = 2,
				type = 'toggle',
				name = "Battleground alerts",
				desc = "Toggles battleground alerts",
				get = function(info)
					return self.db.profile.battleground
				end,
				set = function(info, value)
					self.db.profile.battleground = value
				end,
			},
			instance = {
				order = 3,
				type = 'toggle',
				name = "Instance reset",
				desc = "Reset the kills upon instance change (if not dead)",
				get = function(info)
					return self.db.profile.instance
				end,
				set = function(info, value)
					self.db.profile.instance = value
				end,
			},
			death = {
				order = 4,
				type = 'toggle',
				name = "Death reset",
				desc = "Reset the kills upon death",
				get = function(info)
					return self.db.profile.death
				end,
				set = function(info, value)
					self.db.profile.death = value
				end,
			},
			sound = {
				order = 5,
				type = 'toggle',
				name = "Sound allowed",
				desc = "Toggles all sounds",
				get = function(info)
					return self.db.profile.sound
				end,
				set = function(info, value)
					self.db.profile.sound = value
				end,
			},
			flash = {
				order = 6,
				type = 'toggle',
				name = "Flash allowed",
				desc = "Toggles screen flash",
				get = function(info)
					return self.db.profile.flash
				end,
				set = function(info, value)
					self.db.profile.flash = value
				end,
			},
			slain = {
				order = 7,
				type = 'toggle',
				name = "Slain check",
				desc = "Toggles slain-by check",
				get = function(info)
					return self.db.profile.slain
				end,
				set = function(info, value)
					self.db.profile.slain = value
				end,
			},
			pvp = {
				order = 9,
				type = 'toggle',
				name = "PvP check",
				desc = "Toggles PvP check",
				get = function(info)
					return self.db.profile.pvp
				end,
				set = function(info, value)
					self.db.profile.pvp = value
				end,
			},
		},
	}
	self.options.args.output = self:GetSinkAce3OptionsDataTable()
	LibStub("AceConfig-3.0"):RegisterOptionsTable("KillingSpree", self.options, {"killingspree", "ks"})
	self.optionsDialog = LibStub("AceConfigDialog-3.0")
	self.optionsFrame = self.optionsDialog:AddToBlizOptions("KillingSpree", "KillingSpree")
	self:SetSinkStorage(self.db.profile)
end

function KillingSpree:OnEnable()
	self.kills = 0
	self.dead = false
	self:RegisterEvent("PLAYER_DEAD")
	self:RegisterEvent("ZONE_CHANGED_NEW_AREA")
	self:RegisterEvent("CHAT_MSG_BG_SYSTEM_NEUTRAL")
	self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
end

function KillingSpree:PLAYER_DEAD(event)
	if event == "PLAYER_ALIVE" or event == "PLAYER_UNGHOST" then
		self.dead = false
	elseif event == "PLAYER_DEAD" and not self.dead then
		if self.db.profile.death then self.kills = 0 end
		self.dead = true
	end
end

function KillingSpree:ZONE_CHANGED_NEW_AREA(event)
	self:CancelAllTimers()
	if self.dead then return end
	local instance = select(2, IsInInstance())
	if instance ~= self.instance then
		self.instance = instance
		if self.db.profile.instance then
			self.kills = 0
		end
	end
end

function KillingSpree:CHAT_MSG_BG_SYSTEM_NEUTRAL(event, message)
    if not self.db.profile.battleground then return end
	local bgmessage, bgvictory0, bgvictory1 = string.lower(message), string.lower(VICTORY_TEXT0), string.lower(VICTORY_TEXT1)
	if (self.factiongroup == "Alliance" and string.find(bgmessage, bgvictory1)) or (self.factiongroup == "Horde" and string.find(bgmessage, bgvictory0)) then
		self:Announce({ string.format("%s", self.scheme[self.schemes[self.db.profile.scheme]]['youhavewon'][2]), string.format("Interface\\AddOns\\KillingSpree\\Snds\\%s\\%s", self.schemes[self.db.profile.scheme], self.scheme[self.schemes[self.db.profile.scheme]]['youhavewon'][1]), false })
	elseif (self.factiongroup == "Alliance" and string.find(bgmessage, bgvictory0)) or (self.factiongroup == "Horde" and string.find(bgmessage, bgvictory1)) then
		self:Announce({ string.format("%s", self.scheme[self.schemes[self.db.profile.scheme]]['youhavelost'][2]), string.format("Interface\\AddOns\\KillingSpree\\Snds\\%s\\%s", self.schemes[self.db.profile.scheme], self.scheme[self.schemes[self.db.profile.scheme]]['youhavelost'][1]), false })
	elseif string.find(bgmessage, "30 seconds") or string.find(bgmessage, "thirty seconds") then
	    self:CancelAllTimers()
		self:ScheduleTimer("Announce", 26, { string.format("%s", self.scheme[self.schemes[self.db.profile.scheme]]['count'][5][2]), string.format("Interface\\AddOns\\KillingSpree\\Snds\\%s\\%s", self.schemes[self.db.profile.scheme], self.scheme[self.schemes[self.db.profile.scheme]]['count'][5][1])})
		self:ScheduleTimer("Announce", 27, { string.format("%s", self.scheme[self.schemes[self.db.profile.scheme]]['count'][4][2]), string.format("Interface\\AddOns\\KillingSpree\\Snds\\%s\\%s", self.schemes[self.db.profile.scheme], self.scheme[self.schemes[self.db.profile.scheme]]['count'][4][1])})
		self:ScheduleTimer("Announce", 28, { string.format("%s", self.scheme[self.schemes[self.db.profile.scheme]]['count'][3][2]), string.format("Interface\\AddOns\\KillingSpree\\Snds\\%s\\%s", self.schemes[self.db.profile.scheme], self.scheme[self.schemes[self.db.profile.scheme]]['count'][3][1])})
		self:ScheduleTimer("Announce", 29, { string.format("%s", self.scheme[self.schemes[self.db.profile.scheme]]['count'][2][2]), string.format("Interface\\AddOns\\KillingSpree\\Snds\\%s\\%s", self.schemes[self.db.profile.scheme], self.scheme[self.schemes[self.db.profile.scheme]]['count'][2][1])})
		self:ScheduleTimer("Announce", 30, { string.format("%s", self.scheme[self.schemes[self.db.profile.scheme]]['count'][1][2]), string.format("Interface\\AddOns\\KillingSpree\\Snds\\%s\\%s", self.schemes[self.db.profile.scheme], self.scheme[self.schemes[self.db.profile.scheme]]['count'][1][1])})
	elseif string.find(bgmessage, "15 seconds") or string.find(bgmessage, "fifteen seconds") then
	    self:CancelAllTimers()
		self:ScheduleTimer("Announce", 11, { string.format("%s", self.scheme[self.schemes[self.db.profile.scheme]]['count'][5][2]), string.format("Interface\\AddOns\\KillingSpree\\Snds\\%s\\%s", self.schemes[self.db.profile.scheme], self.scheme[self.schemes[self.db.profile.scheme]]['count'][5][1])})
		self:ScheduleTimer("Announce", 12, { string.format("%s", self.scheme[self.schemes[self.db.profile.scheme]]['count'][4][2]), string.format("Interface\\AddOns\\KillingSpree\\Snds\\%s\\%s", self.schemes[self.db.profile.scheme], self.scheme[self.schemes[self.db.profile.scheme]]['count'][4][1])})
		self:ScheduleTimer("Announce", 13, { string.format("%s", self.scheme[self.schemes[self.db.profile.scheme]]['count'][3][2]), string.format("Interface\\AddOns\\KillingSpree\\Snds\\%s\\%s", self.schemes[self.db.profile.scheme], self.scheme[self.schemes[self.db.profile.scheme]]['count'][3][1])})
		self:ScheduleTimer("Announce", 14, {string.format("%s", self.scheme[self.schemes[self.db.profile.scheme]]['count'][2][2]), string.format("Interface\\AddOns\\KillingSpree\\Snds\\%s\\%s", self.schemes[self.db.profile.scheme], self.scheme[self.schemes[self.db.profile.scheme]]['count'][2][1])})
		self:ScheduleTimer("Announce", 15, { string.format("%s", self.scheme[self.schemes[self.db.profile.scheme]]['count'][1][2]), string.format("Interface\\AddOns\\KillingSpree\\Snds\\%s\\%s", self.schemes[self.db.profile.scheme], self.scheme[self.schemes[self.db.profile.scheme]]['count'][1][1])})
	end
end

function KillingSpree:COMBAT_LOG_EVENT_UNFILTERED(event, timestamp, eventType, sourceGUID, sourceName, sourceFlags, destinationGUID, destinationName, destinationFlags, ...)
	if eventType == "PARTY_KILL" then
	    if self.db.profile.slain and not CombatLog_Object_IsA(sourceFlags, COMBATLOG_FILTER_MINE) then return end
	    if self.db.profile.pvp and not CombatLog_Object_IsA(destinationFlags, COMBATLOG_OBJECT_CONTROL_PLAYER) then return end
	    if not CombatLog_Object_IsA(destinationFlags, COMBATLOG_FILTER_HOSTILE_UNITS) then return end
		self.kills = self.kills + 1
		if self.kills == 1 then
			self:Announce({ string.format("%s", self.scheme[self.schemes[self.db.profile.scheme]]['firstblood'][2]), string.format("Interface\\AddOns\\KillingSpree\\Snds\\%s\\%s", self.schemes[self.db.profile.scheme], self.scheme[self.schemes[self.db.profile.scheme]]['firstblood'][1]), true })
		elseif self.kills > 1 and self.scheme[self.schemes[self.db.profile.scheme]]['multi'][self.kills] then
			self:Announce({ string.format("%s [x%d]", self.scheme[self.schemes[self.db.profile.scheme]]['multi'][self.kills][2], self.kills), string.format("Interface\\AddOns\\KillingSpree\\Snds\\%s\\%s", self.schemes[self.db.profile.scheme], self.scheme[self.schemes[self.db.profile.scheme]]['multi'][self.kills][1]), true })
		elseif self.kills > 1 and self.scheme[self.schemes[self.db.profile.scheme]]['spree'][self.kills] then
			self:Announce({ string.format("%s [x%d]", self.scheme[self.schemes[self.db.profile.scheme]]['spree'][self.kills][2], self.kills), string.format("Interface\\AddOns\\KillingSpree\\Snds\\%s\\%s", self.schemes[self.db.profile.scheme], self.scheme[self.schemes[self.db.profile.scheme]]['spree'][self.kills][1]), true })
		end
	end
end

function KillingSpree:Announce(announcement)
    local message, soundfile, flash = announcement[1], announcement[2], announcement[3]
	if message then
		self:Pour(message, 1, 0, 0)
	end
	if soundfile and self.db.profile.sound then
		PlaySoundFile(soundfile)
	end
	if flash and self.db.profile.flash then
		UIFrameFlash(LowHealthFrame, 0.25, 0.25, 0.5, false, 0, 0)
	end
end

function KillingSpree:Print(message)
   self:Pour(message, 0, 0, 1)
end
